#!/usr/bin/env python
# Copyright 2015-2016 Yelp Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
PaaSTA service list (instances) etc.
"""
from pyramid.response import Response
from pyramid.view import view_config

from paasta_tools.api import settings
from paasta_tools.api.views.exception import ApiFailure
from paasta_tools.cli.utils import get_instance_config
from paasta_tools.kubernetes_tools import KubernetesDeploymentConfig
from paasta_tools.marathon_tools import MarathonServiceConfig


@view_config(route_name="service.autoscaler.get", request_method="GET", renderer="json")
def get_autoscaler_count(request):
    service = request.swagger_data.get("service")
    instance = request.swagger_data.get("instance")
    cluster = settings.cluster
    soa_dir = settings.soa_dir

    instance_config = get_instance_config(service, instance, cluster, soa_dir)
    if not isinstance(
        instance_config, (KubernetesDeploymentConfig, MarathonServiceConfig)
    ):
        error_message = (
            f"Autoscaling is not supported for {service}.{instance} because instance type is not "
            f"marathon or kubernetes."
        )
        raise ApiFailure(error_message, 501)

    response_body = {
        "desired_instances": instance_config.get_instances(),
        "calculated_instances": instance_config.get_instances(with_limit=False),
    }
    return Response(json_body=response_body, status_code=200)


@view_config(
    route_name="service.autoscaler.post", request_method="POST", renderer="json"
)
def update_autoscaler_count(request):
    service = request.swagger_data.get("service")
    instance = request.swagger_data.get("instance")
    cluster = settings.cluster
    soa_dir = settings.soa_dir
    desired_instances = request.swagger_data.get("json_body")["desired_instances"]
    if not isinstance(desired_instances, int):
        error_message = 'The provided body does not have an integer value for "desired_instances": {}'.format(
            request.swagger_data.get("json_body")
        )
        raise ApiFailure(error_message, 500)

    instance_config = get_instance_config(service, instance, cluster, soa_dir, True)
    if not isinstance(
        instance_config, (KubernetesDeploymentConfig, MarathonServiceConfig)
    ):
        error_message = (
            f"Autoscaling is not supported for {service}.{instance} because instance type is not "
            f"marathon or kubernetes."
        )
        raise ApiFailure(error_message, 501)

    max_instances = instance_config.get_max_instances()
    if max_instances is None:
        error_message = f"Autoscaling is not enabled for {service}.{instance}"
        raise ApiFailure(error_message, 404)
    min_instances = instance_config.get_min_instances()

    status = "SUCCESS"
    if desired_instances > max_instances:
        desired_instances = max_instances
        status = (
            "WARNING desired_instances is greater than max_instances %d" % max_instances
        )
    elif desired_instances < min_instances:
        desired_instances = min_instances
        status = (
            "WARNING desired_instances is less than min_instances %d" % min_instances
        )
    try:
        if isinstance(instance_config, KubernetesDeploymentConfig):
            instance_config.set_autoscaled_instances(
                instance_count=desired_instances, kube_client=settings.kubernetes_client
            )
        else:
            instance_config.set_autoscaled_instances(instance_count=desired_instances)
    except Exception as err:
        raise ApiFailure(err, 500)

    response_body = {"desired_instances": desired_instances, "status": status}
    return Response(json_body=response_body, status_code=202)



#!/usr/bin/env python
# Copyright 2015-2016 Yelp Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
PaaSTA flink service list jobs, overview and config.
"""
from pyramid.view import view_config

from paasta_tools.api.views.exception import ApiFailure
from paasta_tools.flink_tools import cr_id
from paasta_tools.flink_tools import curl_flink_endpoint


@view_config(
    route_name="flink.service.instance.jobs", request_method="GET", renderer="json"
)
def list_flink_cluster_jobs(request):
    service = request.swagger_data.get("service")
    instance = request.swagger_data.get("instance")
    try:
        return curl_flink_endpoint(cr_id(service, instance), "jobs")
    except ValueError as e:
        raise ApiFailure(e, 500)


@view_config(
    route_name="flink.service.instance.job_details",
    request_method="GET",
    renderer="json",
)
def get_flink_cluster_job_details(request):
    service = request.swagger_data.get("service")
    instance = request.swagger_data.get("instance")
    job_id = request.swagger_data.get("job_id")
    try:
        return curl_flink_endpoint(cr_id(service, instance), f"jobs/{job_id}")
    except ValueError as e:
        raise ApiFailure(e, 500)


@view_config(
    route_name="flink.service.instance.overview", request_method="GET", renderer="json"
)
def get_flink_cluster_overview(request):
    service = request.swagger_data.get("service")
    instance = request.swagger_data.get("instance")
    try:
        return curl_flink_endpoint(cr_id(service, instance), "overview")
    except ValueError as e:
        raise ApiFailure(e, 500)


@view_config(
    route_name="flink.service.instance.config", request_method="GET", renderer="json"
)
def get_flink_cluster_config(request):
    service = request.swagger_data.get("service")
    instance = request.swagger_data.get("instance")
    try:
        return curl_flink_endpoint(cr_id(service, instance), "config")
    except ValueError as e:
        raise ApiFailure(e, 500)



#!/usr/bin/env python3.7
import argparse
import os
import re

import yaml


def replace(s, values):
    s = re.sub(
        r"<%(.*?)%>",
        lambda x: values.get(
            x.group(0).replace("<%", "").replace("%>", ""), x.group(0)
        ),
        s,
    )
    return re.sub(
        r"\$\((.*?)\)",
        lambda x: os.environ.get(
            x.group(0).replace("$(", "").replace(")", ""), x.group(0)
        ),
        s,
    )


def render_file(src, dst, values):
    basename = os.path.basename(src)
    new_name = replace(basename, values)
    with open(f"{dst}/{new_name}", "w") as new:
        with open(f"{src}", "r") as old:
            new.write(replace(old.read(), values))


def render(src, dst, values={}, exclude={}):
    if os.path.isfile(src):
        render_file(src, dst, values)
        return
    for f in os.scandir(src):
        if f.name.startswith(".") or f.path in exclude:
            continue
        if os.path.isfile(f.path):
            render_file(f.path, dst, values)
        else:
            new_dst = replace(f"{dst}/{f.name}", values)
            try:
                os.makedirs(new_dst, exist_ok=True)
            except OSError as e:
                if e.errno != os.errno.EEXIST:
                    raise
            render(f.path, new_dst, values, exclude)


def parse_args():
    parser = argparse.ArgumentParser(
        description="Replaces all <%%> in all files in src with values provided, and writes the results to dst folder. $() is reserved for environment variables. File/dir that starts with . are ignored"
    )
    parser.add_argument(
        "-s",
        "--src",
        type=str,
        dest="src",
        required=True,
        help="src can be either a valid folder of directory. Note that src directory itself is not rendered. .* files/dirs are ignored.",
    )
    parser.add_argument(
        "-d",
        "--dst",
        type=str,
        dest="dst",
        required=True,
        help="Dst needs to be a directory.",
    )
    parser.add_argument(
        "-v",
        "--values",
        type=str,
        dest="values",
        default=None,
        help="values need to be valid file if provided",
    )
    args = parser.parse_args()
    return args


def render_values(src: str, dst: str, values: str) -> None:
    if values is not None:
        values = os.path.abspath(values)
    # Validate src and values. Dst needs to be a directory. src can be either a valid folder of directory. values need to be valid file if provided.
    if not os.path.exists(src):
        raise Exception("src path is invalid")
    if not os.path.exists(dst) or not os.path.isdir(dst):
        raise Exception("dst path is invalid")
    if values and (not os.path.exists(values) or not os.path.isfile(values)):
        raise Exception("values path is invalid")
    # Lookup for values.yaml in src folder if values is not provided
    if os.path.isdir(src) and values is None and os.path.exists(f"{src}/values.yaml"):
        values = f"{src}/values.yaml"
    config_dict = {}
    if values is not None:
        with open(values) as f:
            config_dict = yaml.safe_load(f)
    # Replace environment variables in values.yaml with environment variables
    for k, v in config_dict.items():
        config_dict[k] = re.sub(
            r"\$\((.*?)\)",
            lambda x: os.environ.get(
                x.group(0).replace("$(", "").replace(")", ""), x.group(0)
            ),
            v,
        )
    render(src, dst, config_dict, {values})


def main():
    args = parse_args()
    src = os.path.abspath(args.src)
    dst = os.path.abspath(args.dst)
    values = args.values

    render_values(src, dst, values)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# Copyright 2015-2016 Yelp Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Usage: ./paasta_execute_docker_command.py [options]

This script will attempt to find a running container that contains an environment variable matching
the specified Mesos task ID and then execute the given command with the specified timeout. The script will
print the output of the command and exit with the same return code as the command.

Command line options:

- -i <MESOS_TASK_ID>, --mesos-id <MESOS_TASK_ID>: Specify a Mesos task ID to search for
- -c <command>, --cmd <command>: Shell command to execute in container
- -t <timeout>, --timeout <timeout>: Timeout for command
"""
import argparse
import signal
import sys
from contextlib import contextmanager

from paasta_tools.mesos_tools import get_container_id_for_mesos_id
from paasta_tools.utils import get_docker_client


def parse_args():
    parser = argparse.ArgumentParser(
        description="Executes given command in Docker container for given Mesos task ID"
    )
    parser.add_argument("-i", "--mesos-id", required=True, help="Mesos task ID")
    parser.add_argument(
        "-c", "--cmd", required=True, help="command to execute in container"
    )
    parser.add_argument(
        "-t", "--timeout", default=45, type=int, help="timeout for command"
    )
    args = parser.parse_args()
    return args


class TimeoutException(Exception):
    pass


@contextmanager
def time_limit(seconds):  # From http://stackoverflow.com/a/601168/1576438
    def signal_handler(signum, frame):
        raise TimeoutException("Timed out!")

    signal.signal(signal.SIGALRM, signal_handler)
    signal.alarm(seconds)
    try:
        yield
    finally:
        signal.alarm(0)


def execute_in_container(docker_client, container_id, cmd, timeout):
    container_info = docker_client.inspect_container(container_id)
    if container_info["ExecIDs"] and len(container_info["ExecIDs"]) > 0:
        for possible_exec_id in container_info["ExecIDs"]:
            exec_info = docker_client.exec_inspect(possible_exec_id)["ProcessConfig"]
            if exec_info["entrypoint"] == "/bin/sh" and exec_info["arguments"] == [
                "-c",
                cmd,
            ]:
                exec_id = possible_exec_id
                break
    else:
        exec_id = docker_client.exec_create(container_id, ["/bin/sh", "-c", cmd])["Id"]
    output = docker_client.exec_start(exec_id, stream=False)
    return_code = docker_client.exec_inspect(exec_id)["ExitCode"]
    return (output, return_code)


def main():
    args = parse_args()

    if not args.mesos_id:
        print(
            "The Mesos task id you supplied seems to be an empty string! Please provide a valid task id."
        )
        sys.exit(2)

    docker_client = get_docker_client()

    container_id = get_container_id_for_mesos_id(docker_client, args.mesos_id)

    if container_id:
        try:
            with time_limit(args.timeout):
                output, return_code = execute_in_container(
                    docker_client, container_id, args.cmd, args.timeout
                )
            print(output)
        except TimeoutException:
            print("Command timed out!")
            return_code = 1
        finally:
            sys.exit(return_code)
    else:
        print("Could not find container with MESOS_TASK_ID '%s'." % args.mesos_id)
        sys.exit(1)


if __name__ == "__main__":
    main()
